@@ -16,6 +16,7 @@ from selenium.common.exceptions import StaleElementReferenceException
 
 from . import __version__ as base_version
 from .utils.selenium_core import SeleniumDriverCore
+from .utils.misc import collect_stack_trace
 
 requests.packages.urllib3.disable_warnings()
 
@@ -49,7 +50,7 @@ class SmartDriver(SeleniumDriverCore):
         return self.driver.get_screenshot_as_base64()
 
 
-    def _classify(self, element_name, is_backup=False):
+    def _classify(self, element_name, is_backup=False, custom_ai_threshold=None):
         msg = ''
         if self.test_case_creation_mode:
             self._test_case_upload_screenshot(element_name)
@@ -99,12 +100,14 @@ class SmartDriver(SeleniumDriverCore):
             source = ''
             # Check results
             try:
+                stack_trace = collect_stack_trace()
                 data = {
                     'screenshot': screenshotBase64,
                     'source': source,
                     'api_key': self.api_key,
                     'label': element_name,
                     'do_exact_match_first': self.do_exact_match_first,
+                    'stack_trace': stack_trace,
                 }
                 start = time.time()
                 response = self.make_json_post_request('/detect', data, self.detect_timeout_message, self.detect_timeout)
