@@ -26,6 +26,7 @@ from selenium.webdriver.common.by import By
 requests.packages.urllib3.disable_warnings()
 from .scanner import Scanner
 
+from .misc import collect_stack_trace
 
 log = logging.getLogger(__name__)
 
@@ -178,7 +179,7 @@ class SeleniumDriverCore(object):
     def scan_domain(self, domain, max_depth=10):
         self.scanner.crawl_domain(domain, max_depth=max_depth)
 
-    def find_element(self, by='id', value=None, element_name=None):
+    def find_element(self, by='id', value=None, custom_ai_threshold=None, element_name=None):
         """
         Find an element given a By strategy and locator.
         :Usage:
@@ -186,17 +187,15 @@ class SeleniumDriverCore(object):
                 element = driver.find_element(By.ID, 'foo')
         :rtype: WebElement
         """
-        # Try to classify with selector
-        #    If success, call update_elem ('train_if_necessary': true)
-        #    If NOT successful, call _classify
-        #        If succesful, return element
-        #        If NOT succesful, raise element not found with link
         if element_name is None:
-            element_name = 'element_name_by_locator_By_%s:_%s' % (str(by).replace('.', '_').replace(' ', '_'), str(value).replace('.', '_').replace(' ', '_'))
+            if by == 'ai':
+                element_name = value
+            else:
+                element_name = 'element_name_by_locator_By_%s:_%s' % (str(by).replace('.', '_').replace(' ', '_'), str(value).replace('.', '_').replace(' ', '_').replace('•',''))
         return self._generic_find_method(
-            self.driver.find_element, element_name, by, value)
+            self.driver.find_element, element_name, by, value, custom_ai_threshold=custom_ai_threshold)
 
-    def find_element_by_accessibility_id(self, accessibility_id, element_name=None):
+    def find_element_by_accessibility_id(self, accessibility_id, element_name=None, custom_ai_threshold=None):
         """
         Finds an element by an accessibility id.
 
@@ -214,9 +213,9 @@ class SeleniumDriverCore(object):
             ::
                 element = driver.find_element_by_accessibility_id('foo')
         """
-        return self.find_element(self, by=By.ACCESSIBILITY_ID, value=accessibility_id, element_name=element_name)
+        return self.find_element(self, by=By.ACCESSIBILITY_ID, value=accessibility_id, element_name=element_name, custom_ai_threshold=custom_ai_threshold)
 
-    def find_element_by_class_name(self, name, element_name=None):
+    def find_element_by_class_name(self, name, element_name=None, custom_ai_threshold=None):
         """
         Finds an element by class name.
 
@@ -234,9 +233,9 @@ class SeleniumDriverCore(object):
             ::
                 element = driver.find_element_by_class_name('foo')
         """
-        return self.find_element(by=By.CLASS_NAME, value=name, element_name=element_name)
+        return self.find_element(by=By.CLASS_NAME, value=name, element_name=element_name, custom_ai_threshold=custom_ai_threshold)
 
-    def find_element_by_css_selector(self, css_selector, element_name=None):
+    def find_element_by_css_selector(self, css_selector, element_name=None, custom_ai_threshold=None):
         """
         Finds an element by css selector.
 
@@ -255,9 +254,9 @@ class SeleniumDriverCore(object):
 
                 element = driver.find_element_by_css_selector('#foo')
         """
-        return self.find_element(by=By.CSS_SELECTOR, value=css_selector, element_name=element_name)
+        return self.find_element(by=By.CSS_SELECTOR, value=css_selector, element_name=element_name, custom_ai_threshold=custom_ai_threshold)
 
-    def find_element_by_id(self, id_, element_name=None):
+    def find_element_by_id(self, id_, element_name=None, custom_ai_threshold=None):
         """
         Finds an element by id.
 
@@ -276,9 +275,9 @@ class SeleniumDriverCore(object):
 
                 element = driver.find_element_by_id('foo')
         """
-        return self.find_element(by=By.ID, value=id_, element_name=element_name)
+        return self.find_element(by=By.ID, value=id_, element_name=element_name, custom_ai_threshold=custom_ai_threshold)
 
-    def find_element_by_link_text(self, link_text, element_name=None):
+    def find_element_by_link_text(self, link_text, element_name=None, custom_ai_threshold=None):
         """
         Finds an element by link text.
 
@@ -297,9 +296,9 @@ class SeleniumDriverCore(object):
 
                 element = driver.find_element_by_link_text('Sign In')
         """
-        return self.find_element(by=By.LINK_TEXT, value=link_text, element_name=element_name)
+        return self.find_element(by=By.LINK_TEXT, value=link_text, element_name=element_name, custom_ai_threshold=custom_ai_threshold)
 
-    def find_element_by_name(self, name, element_name=None):
+    def find_element_by_name(self, name, element_name=None, custom_ai_threshold=None):
         """
         Finds an element by name.
 
@@ -318,9 +317,9 @@ class SeleniumDriverCore(object):
 
                 element = driver.find_element_by_name('foo')
         """
-        return self.find_element(by=By.NAME, value=name, element_name=element_name)
+        return self.find_element(by=By.NAME, value=name, element_name=element_name, custom_ai_threshold=custom_ai_threshold)
 
-    def find_element_by_partial_link_text(self, link_text, element_name=None):
+    def find_element_by_partial_link_text(self, link_text, element_name=None, custom_ai_threshold=None):
         """
         Finds an element by a partial match of its link text.
 
@@ -339,9 +338,9 @@ class SeleniumDriverCore(object):
 
                 element = driver.find_element_by_partial_link_text('Sign')
         """
-        return self.find_element(by=By.PARTIAL_LINK_TEXT, value=link_text, element_name=element_name)
+        return self.find_element(by=By.PARTIAL_LINK_TEXT, value=link_text, element_name=element_name, custom_ai_threshold=custom_ai_threshold)
 
-    def find_element_by_tag_name(self, name, element_name=None):
+    def find_element_by_tag_name(self, name, element_name=None, custom_ai_threshold=None):
         """
         Finds an element by tag name.
 
@@ -360,9 +359,9 @@ class SeleniumDriverCore(object):
 
                 element = driver.find_element_by_tag_name('h1')
         """
-        return self.find_element(by=By.TAG_NAME, value=name, element_name=element_name)
+        return self.find_element(by=By.TAG_NAME, value=name, element_name=element_name, custom_ai_threshold=custom_ai_threshold)
 
-    def find_element_by_xpath(self, xpath, element_name=None):
+    def find_element_by_xpath(self, xpath, element_name=None, custom_ai_threshold=None):
         """
         Finds an element by xpath.
 
@@ -381,9 +380,9 @@ class SeleniumDriverCore(object):
 
                 element = driver.find_element_by_xpath('//div/td[1]')
         """
-        return self.find_element(by=By.XPATH, value=xpath, element_name=element_name)
+        return self.find_element(by=By.XPATH, value=xpath, element_name=element_name, custom_ai_threshold=custom_ai_threshold)
 
-    def find_by_ai(self, element_name):
+    def find_by_ai(self, element_name, custom_ai_threshold=None):
         """
         Finds an element using the ai.
 
@@ -401,21 +400,21 @@ class SeleniumDriverCore(object):
 
                 element = driver.find_by_element_name('some_label')
         """
-        el, key, msg = self._classify(element_name, is_backup=False)
+        el, key, msg = self._classify(element_name, is_backup=False, custom_ai_threshold=custom_ai_threshold)
 
         if el is None:
             print(msg)
             raise NoElementFoundException(msg)
         return el
 
-    def _generic_find_method(self, find_method, element_name, *args):
+    def _generic_find_method(self, find_method, element_name, *args, custom_ai_threshold=None, **kwargs):
         key = None
         msg = 'ai driver exception'
 
         # Run the standard selector
         try:
-            if len(args) == 2 and args[0] == "ai":
-                driver_element = self.find_by_ai(args[1])
+            if len(args[0]) == 2 and args[0] == "ai":
+                driver_element = self.find_by_ai(args[1], custom_ai_threshold=custom_ai_threshold)
             else:
                 driver_element = find_method(*args)
             if driver_element:
@@ -432,7 +431,7 @@ class SeleniumDriverCore(object):
             log.exception(e)
         except Exception as err:
             # If this happens, then error during the driver call
-            classified_element, key, msg = self._classify(element_name)
+            classified_element, key, msg = self._classify(element_name, custom_ai_threshold=custom_ai_threshold)
             if classified_element:
                 log.error('Selector failed, using ai classifier element')
                 return classified_element
@@ -547,7 +546,7 @@ class SeleniumDriverCore(object):
                     return None
 
     def _check_screenshot_exists(self, screenshot_uuid, element_name):
-        data = {'api_key': self.api_key, 'screenshot_uuid': screenshot_uuid, 'label': element_name}
+        data = {'api_key': self.api_key, 'screenshot_uuid': screenshot_uuid, 'label': element_name, 'stack_trace': collect_stack_trace()}
         start = time.time()
         r = self.make_json_post_request('/exists_screenshot', data, self.misc_timeout_message, self.misc_timeout)
         end = time.time()
@@ -631,17 +630,20 @@ class SeleniumDriverCore(object):
         except Exception:
             log.exception('Error checking cached screenshot / uploading it from remote')
 
-    def _test_case_get_box(self, label, event_id=None):
+    def _test_case_get_box(self, label, event_id=None, custom_ai_threshold=None):
         """
             Checks for a bounding box given the last screenshot uuid that we got when uploading it.
         """
         box = None
+        stack_trace = collect_stack_trace()
         data = {'api_key': self.api_key, 'label': label,
                 'screenshot_uuid': self.last_test_case_screenshot_uuid,
                 'run_classifier': self.use_classifier_during_creation,
                 'event_id': event_id,
                 'do_exact_match_first': self.do_exact_match_first,
                 'exact_match_first_threshold': self.exact_match_first_threshold,
+                'stack_trace': stack_trace,
+                'custom_ai_threshold': custom_ai_threshold,
                 }
 
         needs_reload = False
